package ru.dbotthepony.mc.otm.block

import net.minecraft.core.BlockPos
import net.minecraft.world.item.context.BlockPlaceContext
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.EntityBlock
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.entity.BlockEntityTicker
import net.minecraft.world.level.block.entity.BlockEntityType
import net.minecraft.world.level.block.state.BlockState
import ru.dbotthepony.mc.otm.addPreWorldTickerOnce
import ru.dbotthepony.mc.otm.block.entity.StorageExporterBlockEntity
import ru.dbotthepony.mc.otm.block.entity.StorageImporterBlockEntity
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.unaryMinus

class StorageImporterBlock : RotatableMatteryBlock(), EntityBlock {
	override val hasFreeRotation: Boolean get() = true

	override fun newBlockEntity(p_153215_: BlockPos, p_153216_: BlockState): BlockEntity {
		return StorageImporterBlockEntity(p_153215_, p_153216_)
	}

	override fun <T : BlockEntity?> getTicker(
		p_153212_: Level,
		p_153213_: BlockState,
		p_153214_: BlockEntityType<T>
	): BlockEntityTicker<T>? {
		if (p_153212_.isClientSide || p_153214_ !== MBlockEntities.STORAGE_IMPORTER)
			return null

		return BlockEntityTicker { _, _, _, tile -> if (tile is StorageImporterBlockEntity) tile.tick() }
	}

	override fun getStateForPlacement(context: BlockPlaceContext): BlockState? {
		return super.getStateForPlacement(context)?.setValue(FACING_FULL, -context.clickedFace)
	}

	override fun neighborChanged(
		state: BlockState,
		level: Level,
		pos: BlockPos,
		sender: Block,
		sender_pos: BlockPos,
		flag: Boolean
	) {
		super.neighborChanged(state, level, pos, sender, sender_pos, flag)

		if (!level.isClientSide) {
			val tile = level.getBlockEntity(pos)

			if (tile is StorageImporterBlockEntity) {
				addPreWorldTickerOnce(level) {
					tile.checkSurroundings()
				}
			}
		}
	}
}

class StorageExporterBlock : RotatableMatteryBlock(), EntityBlock {
	override val hasFreeRotation: Boolean get() = true

	override fun newBlockEntity(p_153215_: BlockPos, p_153216_: BlockState): BlockEntity {
		return StorageExporterBlockEntity(p_153215_, p_153216_)
	}

	override fun <T : BlockEntity?> getTicker(
		p_153212_: Level,
		p_153213_: BlockState,
		p_153214_: BlockEntityType<T>
	): BlockEntityTicker<T>? {
		if (p_153212_.isClientSide || p_153214_ !== MBlockEntities.STORAGE_EXPORTER)
			return null

		return BlockEntityTicker { _, _, _, tile -> if (tile is StorageExporterBlockEntity) tile.tick() }
	}

	override fun getStateForPlacement(context: BlockPlaceContext): BlockState? {
		return super.getStateForPlacement(context)?.setValue(FACING_FULL, -context.clickedFace)
	}

	override fun neighborChanged(
		state: BlockState,
		level: Level,
		pos: BlockPos,
		sender: Block,
		sender_pos: BlockPos,
		flag: Boolean
	) {
		super.neighborChanged(state, level, pos, sender, sender_pos, flag)

		if (!level.isClientSide) {
			val tile = level.getBlockEntity(pos)

			if (tile is StorageExporterBlockEntity) {
				addPreWorldTickerOnce(level) {
					tile.checkSurroundings()
				}
			}
		}
	}
}
